/***************************************************************************
DSP Handler.

Takes tloopinfo and tfileinfo opens sound out and plays accordingly.

include dsp.h to use.

Use player(tloopinfo *loop, tfileinfo *info) via a pthread_create.  
Kill accordingly or it will die at end of sample if sample is not looped.
***************************************************************************/
#include <malloc.h>
#include <unistd.h>
#ifdef WITH_ESD
#include <esd.h>
#endif
#include <sys/soundcard.h>
#include <pthread.h> 
#include <fcntl.h>
#include <limits.h>
#include <sys/ioctl.h>
#include "defs.h"
#define DSP_BUF_SIZE 88200

#ifdef WITH_ESD
#define BUF_SIZE 	ESD_BUF_SIZE 
#else
#define BUF_SIZE 	DSP_BUF_SIZE 
#endif
typedef signed short        SINT16;                 /* 16 bits signed */
#define AS_S16(bufptr) ((SINT16 *) bufptr)
#define SATURATE_SSHORT(argshort) (\
	((argshort) > (signed long) SHRT_MAX) ? \
		SHRT_MAX \
	  :	((argshort) < (signed long) SHRT_MIN) ?\
			SHRT_MIN \
		  : (signed short) (argshort) \
	)
FILE *recfp = NULL;
tfileinfo recinfo;
int rec = -1;

/**************************************************************************
* Functions to handle either esd or dsp  
**************************************************************************/
void volumechange(void *buf,float volume,int count)
{
	signed long int si32;
	size_t i;
	for (i = 0; i < (count / 2); i++){
		si32 = volume * AS_S16(buf)[i];
		AS_S16(buf)[i] = SATURATE_SSHORT (si32);
	}
}

#ifdef WITH_ESD
int open_sock(tfileinfo *finfo)
{
    char *host = NULL;
	char name[10];
	int rate = finfo->srate;
    int sock = -1;
    int bits = ESD_BITS8, channels = ESD_MONO;
    int mode = ESD_STREAM, func = ESD_PLAY ;
    esd_format_t format = 0;

	if(finfo->depth == 16)
		bits = ESD_BITS16;
  	if(finfo->chans == 2)
		channels = ESD_STEREO;
    format = bits | channels | mode | func;
	sprintf(name,"%i",(int)finfo->datalen);
    sock = esd_play_stream_fallback( format, rate, host, name );
	return (sock);

}

#else

int open_sock(tfileinfo *finfo){
	int i, p;
	int dsp_fd;
	int bsize = BUF_SIZE;

	dsp_fd = open("/dev/dsp", O_WRONLY, 0);

	if (dsp_fd){
	ioctl(dsp_fd, SNDCTL_DSP_SYNC, 0);
	ioctl(dsp_fd, SNDCTL_DSP_RESET, 0);

	p =  finfo->depth;
	i =  ioctl(dsp_fd, SOUND_PCM_WRITE_BITS, &p);

	p =  finfo->chans;
	i += ioctl(dsp_fd, SOUND_PCM_WRITE_CHANNELS, &p);

	p =  finfo->srate;
	i += ioctl(dsp_fd, SOUND_PCM_WRITE_RATE, &p);

	i += ioctl(dsp_fd, SNDCTL_DSP_GETBLKSIZE, &bsize);
	ioctl(dsp_fd, SNDCTL_DSP_SYNC, 0);
	}
	return(dsp_fd);
}
#endif

void player( tfileinfo *info, tloopinfo *loop )
{
	long len;
	long pos;
	void *buffer;
  	int count = BUF_SIZE ;
	int dsp_fd = -1;

	void clean_up(void * arg)
	{
		close (dsp_fd);
		loop->playing=-1;
		if(loop->buf == NULL)
			free(buffer);
	}

	pthread_cleanup_push(clean_up,NULL);	
	if(( dsp_fd = open_sock(info)))
    {

		loop->playing=1;
		if(loop->buf != NULL)
			pos = 0;
		else{
			buffer = malloc(BUF_SIZE);
			pos = ftell(loop->fp);
		}

		do{
			if (loop->buf != NULL){
				len = (loop->endpos - loop->startpos)*info->blkalign ;
				buffer = &loop->buf[0] ;
				if(loop->vol != 1.0){
					volumechange(buffer, loop->vol, len);
				}
				write( dsp_fd, buffer, len ) ;
			}

			else{
				while( pos < loop->endpos*info->blkalign ){
					
					if(loop->endpos*info->blkalign - pos < BUF_SIZE)
						count = loop->endpos*info->blkalign - pos;
					else
						count = BUF_SIZE;
					count =  loop->read(loop->fp, buffer, count);
		/*
					if(loop->vol != 1.0){
						volumechange(buffer, loop->vol, count);
					}
		*/
					write(dsp_fd, buffer, count);
					if (rec != -1)
						recinfo.datalen += fwrite(buffer, count,  1, recfp);
					pos = ftell (loop->fp);
				}

				if ( fseek( loop->fp,loop->startpos*info->blkalign ,SEEK_SET ) == 0)
					pos = loop->startpos*info->blkalign ;	
			}
		}
		while(loop->loop==LOOP);
	}
	pthread_cleanup_pop(1);	
}
