/***************************************************************************
* Snuggles - by Darrick Servis
*
* Snuggles comes under the GNU General Public License. Please
* read the COPYING notice in this distribution.
*
* ================================================================
* The main file.
***************************************************************************/
#include <curses.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <sys/time.h>
#include <pthread.h>
#include "defs.h"
#include "dir.h"
#include "interface.h"
#include "misc.h"

int ferr_val;
int ch,maxy,has_colours = 0;

extern pthread_mutex_t lock;

//tsong snug_song[MAX_SONGS];

char song_directory[1024];
char sample_directory[1024];
char save_directory[1024];

char session_filename[256]; /*posix length ?*/
char sam_filename[256];
extern char *fullpath[MAX_SONGS];
unsigned char song_fpos_real = 0;
unsigned char song_fpos_hl = 0;
unsigned char song_fpos_hl_old = 0;
unsigned char sam_fpos_real = 0;
unsigned char sam_fpos_hl = 0;
unsigned char sam_fpos_hl_old = 0;
char edit_mode;
#define SNUGNAME_SIZE 50
char filename[SNUGNAME_SIZE + 1];
char menu_option = 0;
WINDOW *looppad;
WINDOW *timerpad;

void init_hot_keys(void){
	int x;
	for(x=0;x<48;x++){
		hot_key[x] = -1;
	}
	for(x=49;x<57;x++){
		hot_key[x] = x-49;
	}
	for(x=58;x<127;x++){
		hot_key[x] = -1;
	}
}
char *ferr_messages[] =
{
  "Operation successful.",                              /*0*/
  "Fatal memory allocation error (no memory left).",    /*1*/
  "File open error.",                                   /*2*/
  "Fatal file read error (pretty serious shit).",       /*3*/
  "Unrecognised Module signature.",                     /*4*/
  "Song is corrupt!!!!!.",                              /*5*/
  "Song exceeds set pattern limit.",                    /*6*/
  "Warning: R1 Funk module. Some cmds arranged & added (RTFM please).", /*7*/
  "File create error.",                                 /*8*/
  "Fatal file write error (pretty serious shit).",      /*9*/
  "Can't save module with empty Sequence table.",       /*10*/
  "Unknown samples format.",                            /*11*/
  "Module too complex to be saved as .MOD format.",     /*12*/
  "Unknown song file. Can't make out extension type.",  /*13*/
  "Already at precision. Conversion not necessary."     /*14*/
};
/***************************************************************************
*
***************************************************************************/
void init_colour_pairs(void)
{
  init_pair(COL_NOTHING,COLOR_BLACK,COLOR_BLACK);
  init_pair(COL_TOPBAR,COL_TOPBAR_FG,COL_TOPBAR_BG);
  init_pair(COL_BOX,COL_BOX_FG,COL_BOX_BG);
  init_pair(COL_FERR_MESSAGE,COL_FERR_MESSAGE_FG,COL_FERR_MESSAGE_BG);
  init_pair(COL_TITLE,COL_TITLE_FG,COL_TITLE_BG);
  init_pair(COL_TEXT,COL_TEXT_FG,COL_TEXT_BG);
  init_pair(COL_TEXT2,COL_TEXT2_FG,COL_TEXT2_BG);
  init_pair(COL_BAR,COL_BAR_FG,COL_BAR_BG);
  if(has_colors())
    has_colours = 1;
}

void set_colour_hl(int colour)
{
  if(has_colours)
    attrset(COLOR_PAIR(colour) | A_BOLD);
  else
    attron(A_REVERSE);
}

void set_colour(int colour)
{
  if(has_colours)
    attrset(COLOR_PAIR(colour));
  else
    attroff(A_REVERSE);
}

/***************************************************************************
*
***************************************************************************/
void display_topbar(void)
{
  set_colour(COL_NOTHING);
  clear();
  set_colour_hl(COL_TOPBAR);
  move(0,0);
  addstr(" Snuggles " SNUG_VERSION "           Copyright Infringement is your Best Entertainment Value ");
}

/***************************************************************************
* Utility functions
***************************************************************************/

/************************************
*
************************************/
void hl_bar(int option,int x)
{
  if(option == x)
    attron(A_REVERSE);
  else
    attroff(A_REVERSE);
}
void update_screen(void)
{
  attroff(A_REVERSE);
	pthread_mutex_lock(&lock);
  move(0,0);
  refresh();
pthread_mutex_unlock(&lock);
}
/************************************
*
************************************/
void get_string(int y, int x,char *str,int maxlen)
{
  register int a;

  attron(A_REVERSE);
  for(a = 0;a < maxlen;a++)
  {
    move(y,x + a);
    addch(' ');
  }
  echo();
  move(y,x);
  wgetnstr(stdscr,str,maxlen);
  noecho();
}

/************************************
*
************************************/
void clear_area(int y1,int x1,int y2,int x2)
{
  register int a;
  for(y1 = y1;y1 <= y2;y1++)
    for(a = x1;a <= x2;a++)
    {
      move(y1,a);
      addch(' ');
    }
}

/************************************
* miwi 27/05/96
************************************/
void ferr_message(char *mess_str)
{
  register int amaxy = maxy - 1;

  set_colour(COL_FERR_MESSAGE);
  attron(A_BLINK);
  move(amaxy,0);
  addstr(mess_str);
  addstr(" [hit key]");
  attroff(A_BLINK);
  beep();
  update_screen();
  getch();
  clear_area(amaxy,0,amaxy,79);
  update_screen();
}

/***************************************************************************
****************************************************************************
****************************************************************************
*
* Main screen
*
****************************************************************************
****************************************************************************
***************************************************************************/

/***************************************************************************
*
***************************************************************************/
void main_editor_dis_bg(void)
{
  switch(edit_mode)
  {
    case EM_SNUG:
      snug_display_bg();
      snug_display_all();
      break;
  }
  update_screen();
update_pad();
}

/***************************************************************************
* miwi 27/05/96
***************************************************************************/

void save_snug(void)
{

  if(chdir(song_directory) == -1)
    ferr_message("Couldn't locate song directory.");
  else
  {
    save_session(session_filename);
    if(ferr_val == FERR_OK)
    {
      move(maxy - 2,0);
      addstr(session_filename);
      addstr(" saved..");
      update_screen();
    }
    else
      ferr_message("Error saving session.");
 } 
}

/***************************************************************************
* miwi 27/05/96
***************************************************************************/
int quit_confirm(void)
{
  register int amaxy = maxy -1;

  set_colour(COL_FERR_MESSAGE);
  attron(A_BLINK);
  clear_area(amaxy,0,amaxy,79);
  move(amaxy,0);
  addstr("Quit to main menu yes? [y/N]");
  attroff(A_BLINK);
  beep();
  update_screen();
  ch = getch();
  clear_area(amaxy,0,amaxy,79);
  update_screen();
  if((ch == 'y') || (ch == 'Y'))
    return 1;
  ch = 0;
  return 0;
}

/***************************************************************************
*
***************************************************************************/
void main_editor_loop(void)
{

/* pe_copy_buffer = malloc(64 * MAXIMUM_CHANNELS * sizeof(tslot));
  if(pe_copy_buffer != NULL)
  {*/
    edit_mode = EM_SNUG;
/*snuggle editor*/
    snug_pos_real = 0;
    snug_pos_hl = 0;
    snug_pos_hl_old = 0;
    snug_sd_option = 0;

    main_editor_dis_bg();
    while(edit_mode != EM_QUIT)
    {
      ch = getch();
      switch(edit_mode)
      {
        case EM_SNUG:
          snug_main();
          break;
      }
    }
}

/***************************************************************************
* miwi 27/05/96
***************************************************************************/

void ms_new_file(void)
{

	strcpy(filename,"new_snug");

	if(strlen(filename))
    {
		strcpy(session_filename,filename);
        strcat(session_filename,SNUG_EXTENSION);
        main_editor_loop();
    }
}

void global_keys(void)
{
	switch(ch){
    case FC_SCREEN_ESCAPE:
		if(quit_confirm())
      		edit_mode = EM_QUIT;
		else
      		main_editor_dis_bg();
		ch=0;
      break;
    case FC_SAVE_SNUG:
      save_snug();
		ch=0;
      break;
    case FC_LOAD_SESSION:
		hold = TRUE;
      if(get_session_file_name(song_directory,&song_fpos_real,&song_fpos_hl,
              &song_fpos_hl_old,session_filename) != -1)
            {
              if(strstr(session_filename,".snug") != NULL) 
              {
                load_session(0,session_filename);
                if(ferr_val != FERR_OK)
                  ferr_message(ferr_messages[ferr_val]);
              }
              else
                ferr_message(ferr_messages[FERR_SNGUNKNOWN]);
            }
			hold=FALSE;
      		main_editor_dis_bg();
		ch=0;
            break;
    case FC_CHANGE_DIR: /*change song directory*/
		hold=TRUE;
       get_save_file_name(save_directory,&song_fpos_real,&song_fpos_hl,
           &song_fpos_hl_old,session_filename);
		hold=FALSE;
      main_editor_dis_bg();
		ch=0;
		
       break;
	}
}
/***************************************************************************
*
***************************************************************************/
void ms_main(void)
{
  edit_mode = EM_MAIN;
  while(edit_mode != EM_QUIT)
  {
		ms_new_file();
			
  }
}

/***************************************************************************
****************************************************************************
****************************************************************************
*
* main stuff
*
****************************************************************************
****************************************************************************
***************************************************************************/

/***************************************************************************
*
***************************************************************************/
int cmdline(int argc,char *argv[])
{
  register int x;
//  int y;

  for(x = 0;x < argc;x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 'h':
          printf(
            "\nUsage: snuggles <options>\n"
             " -h        Display this help info\n"
             "\n"
          );
          return 0;
      }
    }
  }
  return 1;
}

/***************************************************************************
*
***************************************************************************/
int main(int argc,char *argv[])
{
  printf(WELCOME);
  if(cmdline(argc,argv))
  {
    getcwd(sample_directory,1024);
    getcwd(song_directory,1024);
    getcwd(save_directory,1024);
	init_hot_keys();
    initscr();
    maxy = 0;
    if(LINES >= 24) maxy = LINES;
    if(LINES >= 40) maxy = LINES;
    if(maxy)
    {
        start_color();
        init_colour_pairs();
        cbreak();
        noecho();
        keypad(stdscr,TRUE);
		looppad = newpad(LINES-4,COLS-6);
		timerpad = newpad(LINES-4,5);
        ms_main();
        clear();
        refresh();
        resetterm();
        echo();
        endwin();
    }
    else
    {
       endwin();
       printf("Error: Screen size is too small. Program aborted.\n");
    }
	//remove_samples();
  }
  return 1;
}
